'use strict';

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var initializeDb = function initializeDb(knex) {
  if (!knex) {
    throw new Error('you must initialize the database before');
  }

  return knex.schema.createTableIfNotExists('users', function (table) {
    table.string('id').primary();
    table.string('userId');
    table.string('platform');
    table.enu('gender', ['unknown', 'male', 'female']);
    table.integer('timezone');
    table.string('locale');
    table.timestamp('created_on');
  });
};

module.exports = function (dbLocation) {

  var knex = null;

  var getDb = function getDb() {
    if (knex) {
      return _bluebird2.default.resolve(knex);
    }

    knex = require('knex')({
      client: 'sqlite3',
      connection: { filename: dbLocation },
      useNullAsDefault: true
    });

    return initializeDb(knex).then(function () {
      return knex;
    });
  };

  var saveUser = function saveUser(_ref) {
    var id = _ref.id,
        platform = _ref.platform,
        gender = _ref.gender,
        timezone = _ref.timezone,
        locale = _ref.locale;

    var userId = platform + ':' + id;
    var userRow = {
      id: userId,
      userId: id,
      platform: platform,
      gender: gender || 'unknown',
      timezone: timezone || null,
      locale: locale || null,
      created_on: (0, _moment2.default)(new Date()).format('x')
    };

    return getDb().then(function (knex) {
      var query = knex('users').insert(userRow).where(function () {
        return this.select(knex.raw(1)).from('users').where('id', '=', userId);
      });
      query = query.toString().replace(/^insert/i, 'insert or ignore');

      return knex.raw(query);
    });
  };

  return {
    get: getDb,
    saveUser: saveUser,
    location: dbLocation
  };
};
//# sourceMappingURL=database.js.map