'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _socket = require('socket.io');

var _socket2 = _interopRequireDefault(_socket);

var _socketioJwt = require('socketio-jwt');

var _socketioJwt2 = _interopRequireDefault(_socketioJwt);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _bodyParser = require('body-parser');

var _bodyParser2 = _interopRequireDefault(_bodyParser);

var _ms = require('ms');

var _ms2 = _interopRequireDefault(_ms);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _uuid = require('uuid');

var _uuid2 = _interopRequireDefault(_uuid);

var _util = require('./util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var setupSocket = function setupSocket(app, bp) {
  var server = _http2.default.createServer(app);
  var io = (0, _socket2.default)(server);

  if (bp.botfile.login.enabled) {
    io.use(_socketioJwt2.default.authorize({
      secret: bp.security.getSecret(),
      handshake: true
    }));
  }

  io.on('connection', function (socket) {
    bp.stats.track('socket', 'connected');

    socket.on('event', function (event) {
      bp.events.emit(event.name, event.data, 'client');
    });
  });

  bp.events.onAny(function (event, data, from) {
    if (from === 'client') {
      // we sent this ourselves
      return;
    }
    io.emit('event', {
      name: event,
      data: data
    });
  });

  return server;
};

var serveApi = function serveApi(app, bp) {
  var logsSecret = _uuid2.default.v4();
  var routersConditions = {};
  var maybeApply = function maybeApply(name, fn) {
    return function (req, res, next) {
      var router = req.originalUrl.match(/\/api\/(botpress-[^\/]+).*$/i);
      if (!router) {
        return fn(req, res, next);
      }

      if (!routersConditions[router[1]]) {
        return fn(req, res, next);
      }

      var condition = routersConditions[router[1]][name];
      if (condition === false) {
        next();
      } else if (typeof condition === 'function' && condition(req) === false) {
        next();
      } else {
        return fn(req, res, next);
      }
    };
  };

  app.use(maybeApply('bodyParser.json', _bodyParser2.default.json()));
  app.use(maybeApply('bodyParser.urlencoded', _bodyParser2.default.urlencoded({ extended: true })));

  app.post('/api/login', function (req, res) {
    bp.stats.track('api', 'auth', 'login');
    var result = bp.security.login(req.body.user, req.body.password, req.ip);
    res.send(result);
  });

  app.use('/api/*', maybeApply('auth', authenticationMiddleware(bp)));

  app.get('/api/ping', function (req, res) {
    res.send('pong');
  });

  app.get('/api/modules', function (req, res) {
    var modules = _lodash2.default.map(bp._loadedModules, function (module) {
      return {
        name: module.name,
        homepage: module.homepage,
        menuText: module.settings.menuText || module.name,
        menuIcon: module.settings.menuIcon || 'view_module'
      };
    });
    res.send(modules);
  });

  app.get('/api/middlewares', function (req, res) {
    res.send(bp.middlewares.list());
  });

  app.post('/api/middlewares/customizations', function (req, res) {
    bp.stats.track('api', 'middlewares', 'customizations');
    var middlewares = req.body.middlewares;

    bp.middlewares.setCustomizations(middlewares);
    bp.middlewares.load();
    res.send(bp.middlewares.list());
  });

  app.delete('/api/middlewares/customizations', function (req, res) {
    bp.stats.track('api', 'middlewares', 'customizations');
    bp.middlewares.resetCustomizations();
    bp.middlewares.load();
    res.send(bp.middlewares.list());
  });

  app.get('/api/notifications', function (req, res) {
    res.send(bp.notifications.load());
  });

  app.get('/api/bot/information', function (req, res) {
    res.send(bp.about.getBotInformation());
  });

  app.get('/api/module/all', function (req, res) {
    bp.modules.listAllCommunityModules().then(function (modules) {
      return res.send(modules);
    });
  });

  app.get('/api/module/popular', function (req, res) {
    bp.modules.listPopularCommunityModules().then(function (popular) {
      return res.send(popular);
    });
  });

  app.get('/api/module/featured', function (req, res) {
    bp.modules.listFeaturedCommunityModules().then(function (featured) {
      return res.send(featured);
    });
  });

  app.get('/api/module/hero', function (req, res) {
    bp.modules.getRandomCommunityHero().then(function (hero) {
      return res.send(hero);
    });
  });

  app.get('/api/bot/information', function (req, res) {
    res.send(bp.bot.getInformation());
  });

  app.get('/api/bot/production', function (req, res) {
    res.send(!_util2.default.isDeveloping);
  });

  app.get('/api/bot/contributor', function (req, res) {
    res.send(bp.bot.getContributor());
  });

  app.get('/api/license', function (req, res) {
    res.send(bp.licensing.getLicenses());
  });

  app.post('/api/license', function (req, res) {
    bp.stats.track('api', 'license', 'change');
    bp.licensing.changeLicense(req.body.license).then(function () {
      res.sendStatus(200);
    }).catch(function (err) {
      return res.status(500).send({
        message: err && err.message
      });
    });
  });

  app.post('/api/module/install/:name', function (req, res) {
    bp.stats.track('api', 'modules', 'install');
    var name = req.params.name;

    bp.modules.install(name).then(function () {
      res.sendStatus(200);
      bp.restart(1000);
    }).catch(function (err) {
      return res.status(500).send({
        message: err && err.message
      });
    });
  });

  app.delete('/api/module/uninstall/:name', function (req, res) {
    bp.stats.track('api', 'modules', 'uninstall');
    var name = req.params.name;

    bp.modules.uninstall(name).then(function () {
      res.sendStatus(200);
      bp.restart(1000);
    }).catch(function (err) {
      return res.status(500).send({
        message: err && err.message
      });
    });
  });

  app.get('/api/logs', function (req, res) {
    var options = {
      from: new Date() - 7 * 24 * 60 * 60 * 1000,
      until: new Date(),
      limit: req.query && req.query.limit || 50,
      start: 0,
      order: 'desc',
      fields: ['message', 'level', 'timestamp']
    };
    bp.logger.query(options, function (err, results) {
      if (err) return console.log(err);
      res.send(results.file);
    });
  });

  app.get('/api/logs/key', function (req, res) {
    res.send({ secret: logsSecret });
  });

  app.get('/logs/archive/:key', function (req, res) {
    bp.stats.track('api', 'logs', 'archive');
    if (req.params.key !== logsSecret) {
      return res.sendStatus(403);
    }

    bp.logger.archiveToFile().then(function (archivePath) {
      logsSecret = _uuid2.default.v4();
      res.download(archivePath);
    });
  });

  var routers = {};
  bp.getRouter = function (name, conditions) {
    if (!routers[name]) {
      var router = _express2.default.Router();
      routers[name] = router;
      app.use('/api/' + name + '/', router);
    }

    if (conditions) {
      routersConditions[name] = Object.assign(routersConditions[name] || {}, conditions);
    }

    return routers[name];
  };
};

var serveStatic = function serveStatic(app, bp) {
  var _loop = function _loop(name) {
    var module = bp._loadedModules[name];
    var bundlePath = _path2.default.join(module.root, module.settings.webBundle || 'bin/web.bundle.js');
    var requestPath = '/js/modules/' + name + '.js';

    app.use(requestPath, function (req, res) {
      try {
        var content = _fs2.default.readFileSync(bundlePath);
        res.contentType('text/javascript');
        res.send(content);
      } catch (err) {
        bp.logger.warn('Could not serve module [' + name + '] at: ' + bundlePath);
      }
    });
  };

  for (var name in bp._loadedModules) {
    _loop(name);
  }

  app.use('/js/env.js', function (req, res) {
    var _bp$botfile$login = bp.botfile.login,
        tokenExpiry = _bp$botfile$login.tokenExpiry,
        enabled = _bp$botfile$login.enabled;

    var optOutStats = !!bp.botfile.optOutStats;
    res.contentType('text/javascript');
    res.send('(function(window) {\n      window.NODE_ENV = "' + (process.env.NODE_ENV || 'development') + '";\n      window.DEV_MODE = ' + _util2.default.isDeveloping + ';\n      window.AUTH_ENABLED = ' + enabled + ';\n      window.AUTH_TOKEN_DURATION = ' + (0, _ms2.default)(tokenExpiry) + ';\n      window.OPT_OUT_STATS = ' + optOutStats + ';\n    })(window || {})');
  });

  app.use(_express2.default.static(_path2.default.join(__dirname, '../lib/web')));

  app.get('*', function (req, res, next) {
    if (/html/i.test(req.headers.accept)) {
      return res.sendFile(_path2.default.join(__dirname, '../lib/web/index.html'));
    }
    next();
  });

  return _bluebird2.default.resolve(true);
};

var authenticationMiddleware = function authenticationMiddleware(bp) {
  return function (req, res, next) {
    if (!bp.botfile.login.enabled) {
      return next();
    }

    if (bp.security.authenticate(req.headers.authorization)) {
      next();
    } else {
      res.status(401).location('/login').end();
    }
  };
};

var WebServer = function () {
  function WebServer(_ref) {
    var botpress = _ref.botpress;

    _classCallCheck(this, WebServer);

    this.bp = botpress;
  }

  _createClass(WebServer, [{
    key: 'start',
    value: function start() {
      var _this = this;

      var app = (0, _express2.default)();
      var server = setupSocket(app, this.bp);
      var port = this.bp.botfile.port || 3000;
      serveApi(app, this.bp);
      serveStatic(app, this.bp);

      server.listen(port, function () {
        _this.bp.events.emit('ready');
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = _lodash2.default.values(_this.bp._loadedModules)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var mod = _step.value;

            mod.handlers.ready && mod.handlers.ready(_this.bp);
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        _this.bp.logger.info(_chalk2.default.green.bold('bot launched, visit: http://localhost:' + port));
      });
    }
  }]);

  return WebServer;
}();

module.exports = WebServer;
//# sourceMappingURL=server.js.map